#include "gego_global_notify.h"

static GegoGlobalNotify * _gego_global_notify_instance = NULL;

#define GEGO_GLOBAL_NOTIFY_GET_PRIVATE(o)\
  G_TYPE_INSTANCE_GET_PRIVATE (o, GEGO_TYPE_GLOBEL_NOTIFY, GegoGlobalNotifyPrivate)

/**
 * section: gego_global_notify
 * @title: GegoGlobalNotify
 * @short_description: subscriber based application wide notifications.
 *
 * The #GegoGlobalNotify tries to solve a probrem that occurs in many peaces of
 * software: How to notify many parts, or components, of changes, or how to
 * send information from one part of a program to many other parts the same
 * program
 **/

/**
 * GegoGlobalNotify:
 * @short_description: aoeu
 */


////////////////////////////////////////////////////////////////////////////////

struct GegoGlobalNotifyPrivate {
  /* <--- private ---> */
  GHashTable * hash_table; /* GHashTable<gchar *><GLinkedList<GegoGlobalNotifyCallbackItem*>*>* */
  
};

struct GegoCallbackItem {
  /* <--- private ---> */
  guint subscription_id;
  GQuark key_id;
  gpointer subscriber_data;
  GCallback callback;
};

G_DEFINE_TYPE_WITH_PRIVATE (GegoGlobalNotify, gego_global_notify, G_TYPE_OBJECT);

GegoGlobalNotify *
gego_global_notify_new (void) {
  GegoGlobalNotify * self = g_object_new (GEGO_TYPE_GLOBEL_NOTIFY, NULL, NULL);
  
  return self;
}


static GObject *
gego_global_notify_constructor (GType                   gtype,
                                guint                   n_properties,
                                GObjectConstructParam * properties) {
  static GObject * self = NULL;
  
  
  if (self == NULL) {
    self = G_OBJECT_CLASS (gego_global_notify_parent_class
                           )->constructor (gtype, n_properties, properties);
    g_object_add_weak_pointer (G_OBJECT (self), (gpointer) &self);
    return G_OBJECT (self);
  }
  
  return g_object_ref (G_OBJECT (self));
}

static void
gego_global_notify_class_init (GegoGlobalNotifyClass * klass) {
  GObjectClass * object_class = G_OBJECT_CLASS (klass);
  
  object_class->constructor = gego_global_notify_constructor;
  
}



static void
gego_global_notify_init (GegoGlobalNotify * self) {
  
  GegoGlobalNotifyPrivate * priv = GEGO_GLOBAL_NOTIFY_GET_PRIVATE (self);
  
  /* Note that the hash table should contain linked lists of
   * GegoGlobalNotifyCallbackItems.
   * In template speak:
   * GHashTable<gchar *><GLinkedList<GegoGlobalNotifyCallbackItem *> *>*
   */
  priv->hash_table = g_hash_table_new (g_str_hash, g_str_equal);
  
}

////////////////////////////////////////////////////////////////////////////////

/**
 * gego_global_notify_initalize:
 * @short_description: Should be run at the start of the program.
 *
 * @err: (allow-none): NULL'ed errer to be passed to function.
 */
gboolean
gego_global_notify_initalize (GError * err) {
  if (_gego_global_notify_instance != NULL) {
    err = g_error_new (GEGO_GLOBAL_NOTIFY_ERROR,
                 GEGO_GLOBAL_NOTIFY_ERROR_ALREADY_INITALIZED,
                 "The Global Notify system is already initialized.");
    return;
  }
  _gego_global_notify_instance = gego_global_notify_new ();
}

/**
 * gego_global_notify_unsubscribe:
 * @short_description: Sholud only be run when the program quits.
 *
 * Calling callbacks after this point is an undefined behaviour.
 *
  * @err: (allow-none):NULL'ed errer to be passed to function.
 */
gboolean
gego_global_notify_uninitalize (GError * err) {
  g_object_unref (_gego_global_notify_instance);
}

/**
 * gego_global_notify_add_notification:
 * 
 * Add a notifications to the system.
 * 
 * @name The name of the notification to be added.
 * @err: (allow-none):NULL'ed errer to be passed to function.
 * 
 * return: GUI for that callbacks. negative value on fail.
 */
gint
gego_global_notify_add_notification (gchar * name, GError * err) {
  return 0;
}

/**
 * gego_global_notify_remove_notification:
 * @short_description: removes a notification from the system.
 * 
 * 
 * @name: The name of the notification to be removed from the system.
 * @notification_id: The notification ID that was given when the notification
 * was added to the system.
 * @err: (allow-none):A NULL'ed error to be passed to the...
 */
void
gego_global_notify_remove_notification (gchar * name,
                                        gint notification_id,
                                        GError * err) {
  
}

/**
 * gego_global_notify_subscribe:
 * @short_description: hook up a callback to a signal name.
 *
 * @name: The name of the callback to attash to.
 * @callback: the callback to hookup to the signal.
 *
 *
 * This function is used to hook up a callback to a globla signal. It is
 * is recomended to save the retuned value for when a callback should be
 * unsubscribed to, using the gego_global_notify_unsubscribe() function.
 * 
 * Here is an example of a callback.
 * \[<!-- language="C" -->
 * 
 * \]
 * 
 * returns: the subscription id of the signal.
 */
gint
gego_global_notify_subscribe (gchar * name, GCallback callback, gpointer subscriber_data, GError * err);

/**
 * gego_global_notify_unsubscribe:
 * @short_description: removes a callback from a 
 */
void
gego_global_notify_unsubscribe (gchar * name, gint subscription_id, GError * err);

gint
gego_global_notify_call (gchar * name, gpointer caller_data, GError * err);

/**
 * gego_global_notify_is_initialized:
 * @short_description: Check if Global Notify is initialized.
 *
 * return: #TRUE if Global Notify is initialized, otherwise #FALSE
 */
gboolean
gego_global_notify_is_initialized (void);

/**
 * gego_global_notify_error_quark: (skip)
 */
GQuark
gego_global_notify_error_quark (void) {
  return g_quark_from_static_string ("gego-global-notify-error-quark");
}
